import {
  Box,
  Button,
  DialogActions,
  IconButton,
  Typography,
} from '@suid/material';
import FiberManualRecordIcon from '@suid/icons-material/FiberManualRecord';
import { Component, createEffect, createSignal, Show } from 'solid-js';
import style from './GpxRecord.module.css';
import Dialog from '../dialog';
import { useI18n } from '@solid-primitives/i18n';
import GpxChooser from '../gpx-chooser';
import { currentGpxId } from '../gpx-dialog';
import {
  startBackgroundGeolocation,
  stopBackgroundGeolocation,
} from '../../lib/background-geolocation';
import dispatch from '../../workers/dispatcher-main';
import { emptyTrk } from '../../db/trk';
import getUri from '../../lib/ids';
import { setCurrentLocation } from '../get-location/GetLocation';
import { getState } from '../map';
import { useNavigate } from '@solidjs/router';

declare global {
  var $lastValidLocationTime: number;
}

const GpxRecord: Component<{}> = (props) => {
  const [t] = useI18n();
  const navigate = useNavigate();
  const [state, setState] = createSignal('stopped');
  const [open, setOpen] = createSignal(false);
  const [watcherId, setWatcherId] = createSignal();
  const [gpxId, setGpxId] = createSignal<string>('');
  const [trksegId, setTrksegId] = createSignal<IdTrkseg>();

  const newLocationHandler = async (location: any) => {
    console.log(
      `Location filtering, elapsed time: ${
        location.time - globalThis.$lastValidLocationTime
      }`
    );
    if (location.time - globalThis.$lastValidLocationTime > 15000) {
      globalThis.$lastValidLocationTime = location.time;
      await dispatch({
        action: 'appendTrkpt',
        params: {
          trksegIdObj: trksegId(),
          trkpt: {
            $: {
              lat: location.latitude,
              lon: location.longitude,
            },
            ele: location.altitude,
            time: new Date(location.time).toISOString(),
            extensions: {
              speed: location.speed,
              accuracy: location.accuracy,
            },
          },
        },
      });
      setCurrentLocation({lon:location.longitude, lat:location.latitude});
      const state = getState();
      navigate(
        `/map/${state.provider}/${location.longitude}/${location.latitude}/${state.zoom}/${state.rotation}`
      );
    }
  };

  const handleClickOpen = () => {
    if (state() === 'stopped') {
      setGpxId(currentGpxId());
    }
    setOpen(true);
  };
  const handleClose = () => {
    setOpen(false);
  };
  const startRecordingHandler = async () => {
    const trkId = await dispatch({
      action: 'appendTrk',
      params: { gpxId: gpxId(), trk: emptyTrk },
    });
    const trkIdObj = getUri('trk', trkId);
    setTrksegId({ ...trkIdObj, trkseg: 0 });
    globalThis.$lastValidLocationTime = 0;
    const id = await startBackgroundGeolocation(newLocationHandler, 20);
    setWatcherId(id);
    setState('recording');
    setOpen(false);
  };
  const pauseRecordingHandler = () => {
    stopBackgroundGeolocation(watcherId());
    setState('paused');
    setOpen(false);
  };
  const resumeRecordingHandler = async () => {
    setTrksegId({ ...trksegId()!, trkseg: trksegId()!.trkseg + 1 });
    const id = await startBackgroundGeolocation(newLocationHandler, 20);
    setWatcherId(id);
    setState('recording');
    setOpen(false);
  };
  const stopRecordingHandler = () => {
    stopBackgroundGeolocation(watcherId());
    setState('stopped');
    setOpen(false);
  };

  return (
    <>
      <div class={style.control}>
        <IconButton onClick={handleClickOpen}>
          <FiberManualRecordIcon
            color={
              state() === 'recording'
                ? 'error'
                : state() === 'paused'
                ? 'primary'
                : 'inherit'
            }
          />
        </IconButton>
      </div>
      <Dialog open={open()} title={t('gpxRecord')} closeHandler={handleClose}>
        <Box
          sx={{
            width: '100%',
            paddingTop: '10px',
            paddingBottom: '10px',
          }}
        >
          <GpxChooser
            gpxId={gpxId}
            setGpxId={setGpxId}
            disabled={state() !== 'stopped'}
          />
          <Typography sx={{ paddingTop: '20px' }} variant='h6'>
            {t(`gpx_${state()}`)}
          </Typography>
          <Show when={state() === 'stopped'}>
            <Button
              color='primary'
              variant='contained'
              onClick={startRecordingHandler}
              disabled={gpxId() === 'new'}
              sx={{
                width: '100%',
                margin: '10px',
              }}
            >
              {t('gpxStartRecording')}
            </Button>
          </Show>
          <Show when={state() === 'recording'}>
            <Button
              color='primary'
              variant='contained'
              onClick={pauseRecordingHandler}
              sx={{
                width: '100%',
                margin: '10px',
              }}
            >
              {t('gpxPauseRecording')}
            </Button>
            <Button
              color='warning'
              variant='contained'
              onClick={stopRecordingHandler}
              sx={{
                width: '100%',
                margin: '10px',
              }}
            >
              {t('gpxStopRecording')}
            </Button>
          </Show>
          <Show when={state() === 'paused'}>
            <Button
              color='primary'
              variant='contained'
              onClick={resumeRecordingHandler}
              sx={{
                width: '100%',
                margin: '10px',
              }}
            >
              {t('gpxResumeRecording')}
            </Button>
            <Button
              color='warning'
              variant='contained'
              onClick={stopRecordingHandler}
              sx={{
                width: '100%',
                margin: '10px',
              }}
            >
              {t('gpxStopRecording')}
            </Button>
          </Show>
        </Box>
      </Dialog>
    </>
  );
};

export default GpxRecord;
